--
-- fastnight
-- V1.0.0.0
--
-- @author apuehri
-- @date 25/12/2018
--
-- Copyright (C) apuehri
-- V1.0.0.0 ..... LS19 first implementation

fastnight = {};
fastnight.Version = "1.0.0.0";
fastnight.debug = false;
fastnight.dir = g_currentModDirectory;
fastnight.timeScaling = {1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;20;25;30;35;40;50;60;70;80;90;100;120;140;160;180;200;220;240;260;280;300;};
fastnight.hourSteps = {0;1;2;3;4;5;6;7;8;9;10;11;12;13;14;15;16;17;18;19;20;21;22;23;};
fastnight.minuteSteps = {0;10;20;30;40;50;};

function fastnight.prerequisitesPresent(specializations)
    return true;
end;

function fastnight:loadMap(name)
	print("--- loading FastNight V".. fastnight.Version .. " (c) by aPuehri ---")
    -- Only needed for action event for player
    Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, fastnight.registerActionEventsPlayer);
	Player.removeActionEvents = Utils.appendedFunction(Player.removeActionEvents, fastnight.removeActionEventsPlayer);		
		
	-- SaveSettings
	FSBaseMission.saveSavegame = Utils.appendedFunction(FSBaseMission.saveSavegame, fastnight.saveSettings);		
		
	--initialize
	fastnight.autoDayTimeScale = true;
	fastnight.startDayHour = 6;
	fastnight.startDayMinute = 0;
	fastnight.speedDay = 3;	
	fastnight.autoNightTimeScale = true;
	fastnight.startNightHour = 22;
	fastnight.startNightMinute = 0;
	fastnight.speedNight = 80;
	fastnight.manScaling = 0;
	fastnight.showHelp = true;
	fastnight.eventName = {};
	
	--gui
    fastnight.gui = {};
    fastnight.gui["fnSettingGui"] = fnGui:new();
    g_gui:loadGui(fastnight.dir .. "gui/fnGui.xml", "fnGui", fastnight.gui.fnSettingGui);	
	
	--load Savegame
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
	end;

	if fileExists(savegameFolderPath .. '/careerSavegame.xml') then
		local key = "fastnight";
		if fileExists(savegameFolderPath .. '/fastnight.xml') then
			local xmlFile = loadXMLFile("fastnight", savegameFolderPath .. "/fastnight.xml", key);

			-- AutoDayTimeScale
			fastnight.autoDayTimeScale = Utils.getNoNil(getXMLBool(xmlFile, key.."#autoDayTimeScale"), true);
			
			--startDayHour
			local startDayHour = getXMLInt(xmlFile, key.."#startDayHour");
			if startDayHour ~= nil then
				if startDayHour >= 0 and startDayHour <= fastnight.startNightHour then
					fastnight.startDayHour = startDayHour;
				end;
			end;			

			--startDayMinute
			local startDayMinute = getXMLInt(xmlFile, key.."#startDayMinute");
			if startDayMinute ~= nil then
				if startDayMinute >= 0 and startDayMinute <= 59 then
					fastnight.startDayMinute = startDayMinute;
				end;
			end;	

			--speedDay
			local speedDay = getXMLInt(xmlFile, key.."#speedDay");
			if speedDay ~= nil then
				if speedDay >= 1 and speedDay <= 300 then
					fastnight.speedDay = speedDay;
				end;
			end;	

			-- AutoNightTimeScale
			fastnight.autoNightTimeScale = Utils.getNoNil(getXMLBool(xmlFile, key.."#autoNightTimeScale"), true);			
			
			--startNightHour
			local startNightHour = getXMLInt(xmlFile, key.."#startNightHour");
			if startNightHour ~= nil then
				if startNightHour > startDayHour and startNightHour <= 24 then
					fastnight.startNightHour = startNightHour;
				end;
			end;			

			--startNightMinute
			local startNightMinute = getXMLInt(xmlFile, key.."#startNightMinute");
			if startNightMinute ~= nil then
				if startNightMinute >= 0 and startNightMinute <= 59 then
					fastnight.startNightMinute = startNightMinute;
				end;
			end;	

			--speedNight
			local speedNight = getXMLInt(xmlFile, key.."#speedNight");
			if speedNight ~= nil then
				if speedNight >= 1 and speedNight <= 300 then
					fastnight.speedNight = speedNight;
				end;
			end;			
			
			-- ShowHelp
			fastnight.showHelp = Utils.getNoNil(getXMLBool(xmlFile, key.."#showHelp"), true);			
			
			delete(xmlFile);
		end;
	end;

	-- set fastnight.manScaling
	fastnight.manScaling = g_currentMission.missionInfo.timeScale;
	
	-- update Gui-Data
	fastnight:updGui();
	
end;

function fastnight:loadSavegame() 
end;

function fastnight:registerActionEventsPlayer()
	-- FastNight Minus
	local result, eventName = InputBinding.registerActionEvent(g_inputBinding, 'fastnight_Minus',self, fastnight.actionFastnight_Minus ,false ,true ,false ,true)
	if result then
		table.insert(fastnight.eventName, eventName);
		g_inputBinding.events[eventName].displayIsVisible = fastnight.showHelp;
    end
	-- FastNight Plus	
	local result, eventName = InputBinding.registerActionEvent(g_inputBinding, 'fastnight_Plus',self, fastnight.actionFastnight_Plus ,false ,true ,false ,true)
	if result then
        table.insert(fastnight.eventName, eventName);
		g_inputBinding.events[eventName].displayIsVisible = fastnight.showHelp;
    end
	-- FastNight Setting	
	local result, eventName = InputBinding.registerActionEvent(g_inputBinding, 'fastnight_Setting',self, fastnight.actionFastnight_Setting ,false ,true ,false ,true)
	if result then
		table.insert(fastnight.eventName, eventName);
        g_inputBinding.events[eventName].displayIsVisible = fastnight.showHelp;
    end
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:registerActionEventsPlayer(fastnight.eventName)");
		DebugUtil.printTableRecursively(fastnight.eventName,"----",0,1)
	end;	
end;

function fastnight:removeActionEventsPlayer()
	fastnight.eventName = {};
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:removeActionEventsPlayer(fastnight.eventName)");
		DebugUtil.printTableRecursively(fastnight.eventName,"----",0,1)
	end;
end;

function fastnight:mouseEvent(posX, posY, isDown, isUp, button)
end;

function fastnight:keyEvent(unicode, sym, modifier, isDown)
end;

function fastnight:update(dt)
	-- Auto scaling Day / Night
	if fastnight.autoDayTimeScale then
		if (math.abs(g_currentMission.environment.currentHour) == math.abs(fastnight.startDayHour)) and (math.abs(g_currentMission.environment.currentMinute) == math.abs(fastnight.startDayMinute)) then 
			g_currentMission:setTimeScale(fastnight.speedDay);
			fastnight.manScaling = fastnight.speedDay;
		end;
	end;
	if fastnight.autoNightTimeScale then	
		if (math.abs(g_currentMission.environment.currentHour) == math.abs(fastnight.startNightHour)) and (math.abs(g_currentMission.environment.currentMinute) == math.abs(fastnight.startNightMinute)) then
			g_currentMission:setTimeScale(fastnight.speedNight);
			fastnight.manScaling = fastnight.speedNight;
		end;
	end;
end;

function fastnight:draw()   
end;

function fastnight:saveSavegame() 
end;

function fastnight:delete()
end;

function fastnight:deleteMap()
end;

function fastnight:saveSettings()
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:saveSettings -- Settings will be saved");
	end;
	local savegameIndex = g_currentMission.missionInfo.savegameIndex;
	local savegameFolderPath = g_currentMission.missionInfo.savegameDirectory;
	if savegameFolderPath == nil then
		savegameFolderPath = ('%ssavegame%d'):format(getUserProfileAppPath(), savegameIndex);
		if fastnight.debug then
			print("--- FastNight Debug ... fastnight:saveSettings(savegameFolderPath= " ..tostring(savegameFolderPath)..")");
		end;		
	end;
	
	local key = "fastnight";
	local xmlFile = createXMLFile("fastnight", savegameFolderPath .. "/fastnight.xml", key);
	
	-- AutoDayTimeScale
	setXMLBool(xmlFile, key .. "#autoDayTimeScale",fastnight.autoDayTimeScale);
	--startDayHour		
	setXMLInt(xmlFile, key .. "#startDayHour",fastnight.startDayHour);
	--startDayMinute		
	setXMLInt(xmlFile, key .. "#startDayMinute",fastnight.startDayMinute);
	--speedDay		
	setXMLInt(xmlFile, key .. "#speedDay",fastnight.speedDay);
	-- AutoDayTimeScale
	setXMLBool(xmlFile, key .. "#autoNightTimeScale",fastnight.autoNightTimeScale);		
	--startNightHour		
	setXMLInt(xmlFile, key .. "#startNightHour",fastnight.startNightHour);
	--startNightMinute		
	setXMLInt(xmlFile, key .. "#startNightMinute",fastnight.startNightMinute);
	--speedNight		
	setXMLInt(xmlFile, key .. "#speedNight",fastnight.speedNight);
	-- ShowHelp
	setXMLBool(xmlFile, key .. "#showHelp",fastnight.showHelp);
	
	saveXMLFile(xmlFile);
	delete(xmlFile);
end;

function fastnight:updGui()
	-- Day Start Hour
	local index = fastnight.startDayHour + 1;
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(Day Start Hour index= " ..tostring(index)..")");
	end;
	if (index >= 1) and (index <= #fastnight.hourSteps) then
		fastnight.gui.fnSettingGui:setStartDayHour(index);		
	end;
	
	-- Day Start Minute
	index = 0;
	for i=1, #fastnight.minuteSteps, 1 do
		if (fastnight.minuteSteps[i] == fastnight.startDayMinute) then
			index = i;
			break
		end;
	end;	
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(Day Start Minute index= " ..tostring(index)..")");
	end;
	if (index ~= 0) then
		fastnight.gui.fnSettingGui:setStartDayMinute(index);
	end;
	
	-- Day TimeScale
	index = 0;
	for i=1, #fastnight.timeScaling, 1 do
		if (fastnight.timeScaling[i] == fastnight.speedDay) then
			index = i;
			break
		end;
	end;	
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(Day TimeScale index= " ..tostring(index)..")");
	end;
	if (index ~= 0) then
		fastnight.gui.fnSettingGui:setDayTimeScale(index);
	end;
	
	-- AutoDayTimeScale
	fastnight.gui.fnSettingGui:setAutoDayTimeScale(fastnight.autoDayTimeScale);
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(AutoDayTimeScale= " ..tostring(fastnight.autoDayTimeScale)..")");
	end;	

	-- ShowHelp
	fastnight.gui.fnSettingGui:setShowHelp(fastnight.showHelp);
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(ShowHelp= " ..tostring(fastnight.showHelp)..")");
	end;	
	
	-- Night Start Hour
	local index = fastnight.startNightHour + 1;
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(Night Start Hour index= " ..tostring(index)..")");
	end;
	if (index >= 1) and (index <= #fastnight.hourSteps) then
		fastnight.gui.fnSettingGui:setStartNightHour(index);		
	end;
	
	-- Night Start Minute
	index = 0;
	for i=1, #fastnight.minuteSteps, 1 do
		if (fastnight.minuteSteps[i] == fastnight.startNightMinute) then
			index = i;
			break
		end;
	end;	
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(Night Start Minute index= " ..tostring(index)..")");
	end;
	if (index ~= 0) then
		fastnight.gui.fnSettingGui:setStartNightMinute(index);
	end;
	
	-- Night TimeScale
	index = 0;
	for i=1, #fastnight.timeScaling, 1 do
		if (fastnight.timeScaling[i] == fastnight.speedNight) then
			index = i;
			break
		end;
	end;	
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(Night TimeScale index= " ..tostring(index)..")");
	end;
	if (index ~= 0) then
		fastnight.gui.fnSettingGui:setNightTimeScale(index);
	end;
	
	-- AutoNightTimeScale
	fastnight.gui.fnSettingGui:setAutoNightTimeScale(fastnight.autoNightTimeScale);
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:updgui(AutoNightTimeScale= " ..tostring(fastnight.autoNightTimeScale)..")");
	end;	
end;

function fastnight:settingsFromGui(inddsh, inddsm, inddts, stadts, indnsh, indnsm, indnts, stants, stash)
	fastnight.startDayHour = fastnight.hourSteps[inddsh];
	fastnight.startDayMinute = fastnight.minuteSteps[inddsm];
	fastnight.speedDay = fastnight.timeScaling[inddts];
	fastnight.autoDayTimeScale = stadts;
	fastnight.startNightHour = fastnight.hourSteps[indnsh];
	fastnight.startNightMinute = fastnight.minuteSteps[indnsm];
	fastnight.speedNight = fastnight.timeScaling[indnts];
	fastnight.autoNightTimeScale = stants;
	fastnight.showHelp = stash;
	if fastnight.debug then
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.startDayHour= " ..tostring(fastnight.startDayHour)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.startDayMinute= " ..tostring(fastnight.startDayMinute)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.speedDay= " ..tostring(fastnight.speedDay)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.autoDayTimeScale= " ..tostring(fastnight.autoDayTimeScale)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.startNightHour= " ..tostring(fastnight.startNightHour)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.startNightMinute= " ..tostring(fastnight.startNightMinute)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.speedNight= " ..tostring(fastnight.speedNight)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.autoNightTimeScale= " ..tostring(fastnight.autoNightTimeScale)..")");
		print("--- FastNight Debug ... fastnight:settingsFromGui(fastnight.showHelp= " ..tostring(fastnight.showHelp)..")");
	end;
	--Update Show Help
	for i=1, #fastnight.eventName, 1 do
		if (g_inputBinding.events[fastnight.eventName[i]] ~= nil) then
			g_inputBinding.events[fastnight.eventName[i]].displayIsVisible = fastnight.showHelp;
		end;	
	end;	
end;

function fastnight:actionFastnight_Minus(actionName, keyStatus, arg3, arg4, arg5)
	if (fastnight.manScaling >= 120) then
		fastnight.manScaling = fastnight.manScaling - 20;
	elseif (fastnight.manScaling > 40) and (fastnight.manScaling <= 100) then
		fastnight.manScaling = fastnight.manScaling - 10;
	elseif (fastnight.manScaling > 15) and (fastnight.manScaling <= 40) then
		fastnight.manScaling = fastnight.manScaling - 5;
	elseif (fastnight.manScaling > 1) and (fastnight.manScaling <= 15) then
		fastnight.manScaling = fastnight.manScaling - 1;
	end;
	g_currentMission:setTimeScale(fastnight.manScaling);	
end;

function fastnight:actionFastnight_Plus(actionName, keyStatus, arg3, arg4, arg5)
	if (fastnight.manScaling < 15) then
		fastnight.manScaling = fastnight.manScaling + 1;
	elseif (fastnight.manScaling >= 15) and (fastnight.manScaling < 40) then
		fastnight.manScaling = fastnight.manScaling + 5;
	elseif (fastnight.manScaling >= 40) and (fastnight.manScaling < 100) then
		fastnight.manScaling = fastnight.manScaling + 10;
	elseif (fastnight.manScaling >= 100) and (fastnight.manScaling < 300) then
		fastnight.manScaling = fastnight.manScaling + 20;			
	end;
	g_currentMission:setTimeScale(fastnight.manScaling);	
end;

function fastnight:actionFastnight_Setting(actionName, keyStatus, arg3, arg4, arg5)
	if fastnight.gui.fnSettingGui.isOpen then
		fastnight.gui.fnSettingGui:onClickBack()
	elseif g_gui.currentGui == nil then
		g_gui:showGui("fnGui")
	end;
end;

addModEventListener(fastnight);
